<?php
/*
Plugin Name: BT Amazon
Description: URL, Image, Title etc. von Amazon
Author: Benedicta Thölke
Version: 1.0.0
Author URI: https://www.bt-webdesign.de/
*/
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

// Save parameters to wp_options on plugin activation
function bt_amazon_save_parameters() {

    update_option( 'bt_amazon_param1', 'Partner-Tag / Tracking-ID', 'no' );
    update_option( 'bt_amazon_param2', 'ACCESS-Key', 'no' );
    update_option( 'bt_amazon_param3', 'Secret-Key', 'no' );
    update_option( 'bt_amazon_param4', 'webservices.amazon.de', 'no' );         // https://webservices.amazon.com/paapi5/documentation/common-request-parameters.html#host-and-region
    update_option( 'bt_amazon_param5', 'eu-west-1', 'no' );                     // https://webservices.amazon.com/paapi5/documentation/common-request-parameters.html#host-and-region
}
register_activation_hook( __FILE__, 'bt_amazon_save_parameters' );

$parameters = bt_amazon_get_parameters();                                       // Get BT-Amazon Parameter
define('BT_AMAZON_PARTNERTAG', $parameters['bt_amazon_param1']);                // AMAZON Partner-Tag / Tracking-ID
define('BT_AMAZON_ACCESSKEY', $parameters['bt_amazon_param2']);                 // AMAZON ACCESS-Key
define('BT_AMAZON_SECRETKEY', $parameters['bt_amazon_param3']);                 // AMAZON Secret-Key
define('BT_AMAZON_HOST', $parameters['bt_amazon_param4']);                      // AMAZON Host
define('BT_AMAZON_REGION', $parameters['bt_amazon_param5']);                    // AMAZON Region

/*
 * Get AMAZON-Produkt-URL + AMAZON-Image-URLs, width + height
 *
 * Shortcode-Beispiele:
 * [bt-amazon asin="1234567890"]                                                // Default Image-Size = medium
 * [bt-amazon asin="1234567890" imgsize="large"]                                // imgsize="small|medium|large"
 * [bt-amazon asin="1234567890" affiliate="true"]                               // Affiliate-Hinweis einfügen
 * [bt-amazon asin="1234567890" affiliate="true" imgsize="small"]               // Affiliate-Hinweis einfügen, Image-Size = small
 *
 * Shortcode-Parameter:
 * asin = Amazon Standard Identification Number  (entspricht bei Büchern der 10stelligen ISBN)
 * imgsize = small|medium|large , Default: medium
 *
 * Return:
 * Link zu Amazon-Produkt mit Amazon-Image | leer im Fehlerfall (Fehlermeldung siehe debug.log)
 *
 * Weitere Informationen:
 * https://webservices.amazon.com/paapi5/documentation/index.html
 *
 */
function bt_amazon( $atts ){

  if ( is_array($atts) ){

     $amazon_link = '';
     $imgsize = 'medium';
     if ( array_key_exists('imgsize', $atts) ) $imgsize = $atts['imgsize'];

     if ( array_key_exists('asin', $atts) ){

        if ( array_key_exists('asin', $atts) ){
		       $asin = $atts['asin'];
		       }
        else {
              bt_write_log('BT-Amazon : ASIN fehlt im Shortcode');
              return '';
             }

/*
 * Aufruf Funktion getASINInfo($asin)
 *
 * Returns array
 *
 * error == false
 * ASIN-Daten (AMAZON-URL, AMAZON-Title, AMAZON-Image-URLs mit width und height (small, medium, large)
 *
 * error == true
 * Fehlermeldung
 */
          $ret = getASINInfo($asin);

          if ( $ret['error'] && isset($ret['status_code']) && $ret['status_code'] == '429' ) {        // TooManyRequests
               sleep(1);
               $ret = getASINInfo($asin);
          }
          if ( $ret['error'] && isset($ret['status_code']) && $ret['status_code'] == '429' ) {        // TooManyRequests
               sleep(1);
               $ret = getASINInfo($asin);
          }

          if ( $ret['error'] ) {                                                                      // Abbruch, wenn dreimal nicht erfolgreich
               $amazon_link = '';
			   bt_write_log('BT-Amazon Error: ASIN='.$asin);
               bt_write_log($ret['msg']);
          }
          else {

               $imgurl = '';
               $imgwidth = 0;
               $imgheight = 0;

               if ( $imgsize == 'small' && isset($ret['asin_image_small_url']) && isset($ret['asin_image_small_width']) && isset($ret['asin_image_small_height']) ) {
                   $imgurl = $ret['asin_image_small_url'];
                   $imgwidth = $ret['asin_image_small_width'];
                   $imgheight = $ret['asin_image_small_height'];
               }
               else
               if ( $imgsize == 'large' && isset($ret['asin_image_large_url']) && isset($ret['asin_image_large_width']) && isset($ret['asin_image_large_height']) ) {
                   $imgurl = $ret['asin_image_large_url'];
                   $imgwidth = $ret['asin_image_large_width'];
                   $imgheight = $ret['asin_image_large_height'];
               }
               else
               if ( isset($ret['asin_image_medium_url']) && isset($ret['asin_image_medium_width']) && isset($ret['asin_image_medium_height']) ) {
                   $imgurl = $ret['asin_image_medium_url'];
                   $imgwidth = $ret['asin_image_medium_width'];
                   $imgheight = $ret['asin_image_medium_height'];
               }

               $url = $ret['asin_product_url'];
               $amazon_link = '<a href='.$url.' target="_blank" title="'.$ret['asin_title'].'">';

               if ( array_key_exists('affiliate', $atts) && $atts['affiliate'] == 'true' )
                    $amazon_link .= '<figure style="width: '.$imgwidth.'px;">';

               $amazon_link .= '<img src="'.$imgurl.'" alt="'.$ret['asin_title'].'" style="width: '.$imgwidth.'px; height:'.$imgheight.'px;" >';

               if ( array_key_exists('affiliate', $atts) && $atts['affiliate'] == 'true' ){
                    $amazon_link .= '<figcaption class="wp-caption-text" style="font-size: 70%;"><i>Affiliate-Link zu Amazon*</i></figcaption>';
                    $amazon_link .= '</figure>';
               }

               $amazon_link .= '</a>';
          }
     }
     else {
          bt_write_log('BT-Amazon : ASIN fehlt');
          $amazon_link = '';
     }
  }
  else {
       bt_write_log('BT-Amazon : Parameter fehlen');
       $amazon_link = '';
  }

return $amazon_link;
}
add_shortcode( 'bt-amazon', 'bt_amazon' );

// Retrieve parameters from wp_options
function bt_amazon_get_parameters() {
    $param1 = get_option( 'bt_amazon_param1' );
    $param2 = get_option( 'bt_amazon_param2' );
    $param3 = get_option( 'bt_amazon_param3' );
    $param4 = get_option( 'bt_amazon_param4' );
    $param5 = get_option( 'bt_amazon_param5' );

    return array(
        'bt_amazon_param1' => $param1,
        'bt_amazon_param2' => $param2,
        'bt_amazon_param3' => $param3,
        'bt_amazon_param4' => $param4,
        'bt_amazon_param5' => $param5
    );
}

// Update parameters in wp_options
function bt_amazon_update_parameters( $param1, $param2, $param3, $param4, $param5 ) {

    update_option( 'bt_amazon_param1', $param1 );
    update_option( 'bt_amazon_param2', $param2 );
    update_option( 'bt_amazon_param3', $param3 );
    update_option( 'bt_amazon_param4', $param4 );
    update_option( 'bt_amazon_param5', $param5 );

    add_settings_error( 'bt_amazon_settings', 'parameters_updated', 'Parameter erfolgreich gespeichert', 'updated' );
}

// Add plugin options page
function bt_amazon_add_options_page() {
    add_options_page(
        'BT Amazon Options',
        'BT Amazon Options',
        'manage_options',
        'bt-amazon',
        'bt_amazon_options_page_callback'
    );
}
add_action( 'admin_menu', 'bt_amazon_add_options_page' );

// Render options page
function bt_amazon_options_page_callback() {
    $parameters = bt_amazon_get_parameters();

    if ( isset( $_POST['bt_amazon_update_parameters'] ) ) {
        $param1 = sanitize_text_field( $_POST['bt_amazon_param1'] );
        $param2 = sanitize_text_field( $_POST['bt_amazon_param2'] );
        $param3 = sanitize_text_field( $_POST['bt_amazon_param3'] );
        $param4 = sanitize_text_field( $_POST['bt_amazon_param4'] );
        $param5 = sanitize_text_field( $_POST['bt_amazon_param5'] );

        bt_amazon_update_parameters( $param1, $param2, $param3, $param4, $param5 );
        $parameters = bt_amazon_get_parameters();
    }

    ?>
    <div class="wrap">
        <h1>BT Amazon Options</h1>
        <p>Um das Plugin nutzen zu können, müssen die Daten vom eigenen Amazon.de-Partnerkonto eingefügt werden:</p>
        <p>Weitere Informationen: <a href="https://partnernet.amazon.de/assoc_credentials/home" target="_blank" rel="nofollow">https://partnernet.amazon.de/assoc_credentials/home</a></p>
        <?php settings_errors( 'bt_amazon_settings' ); ?>
        <form method="post" action="">
            <?php wp_nonce_field( 'bt_amazon_update_parameters' ); ?>
            <table class="form-table">
                <tr>
                    <th scope="row"><label for="bt_amazon_param1">AMAZON Partner-Tag / Tracking-ID</label></th>
                    <td><input type="text" id="bt_amazon_param1" name="bt_amazon_param1" value="<?php echo esc_attr( $parameters['bt_amazon_param1'] ); ?>" /></td>
                    <td></td>
                </tr>
                <tr>
                    <th scope="row"><label for="bt_amazon_param2">AMAZON ACCESS-Key</label></th>
                    <td><input type="text" id="bt_amazon_param2" name="bt_amazon_param2" value="<?php echo esc_attr( $parameters['bt_amazon_param2'] ); ?>" /></td>
                    <td></td>
                </tr>
                <tr>
                    <th scope="row"><label for="bt_amazon_param3">AMAZON Secret-Key</label></th>
                    <td><input type="text" id="bt_amazon_param3" name="bt_amazon_param3" value="<?php echo esc_attr( $parameters['bt_amazon_param3'] ); ?>" /></td>
                    <td></td>
                </tr>
                <tr>
                    <th scope="row"><label for="bt_amazon_param4">AMAZON Host</label></th>
                    <td><input type="text" id="bt_amazon_param4" name="bt_amazon_param4" value="<?php echo esc_attr( $parameters['bt_amazon_param4'] ); ?>" /></td>
                    <td><i>Default: webservices.amazon.de (siehe <a href="https://webservices.amazon.com/paapi5/documentation/common-request-parameters.html#host-and-region" target="_blank" rel="nofollow">Amazon Host und Region</a>)</i></td>
                </tr>
                <tr>
                    <th scope="row"><label for="bt_amazon_param5">AMAZON Region</label></th>
                    <td><input type="text" id="bt_amazon_param5" name="bt_amazon_param5" value="<?php echo esc_attr( $parameters['bt_amazon_param5'] ); ?>" /></td>
                    <td><i>Default: eu-west-1 (siehe <a href="https://webservices.amazon.com/paapi5/documentation/common-request-parameters.html#host-and-region" target="_blank" rel="nofollow">Amazon Host und Region</a>)</i></td>
                </tr>
            </table>
            <?php submit_button( 'speichern' ); ?>
            <input type="hidden" name="bt_amazon_update_parameters" value="1" />
        </form>
    </div>
    <?php
}

/*
 * function bt_write_log
 *
 * Input: $text (char|int|array|obj)
 * Output: $text in debug.log
 */
if (!function_exists('bt_write_log')) {
function bt_write_log($text) {
  if (true === WP_DEBUG) {
      if (is_array($text) || is_object($text)) {
          error_log(print_r($text, true));
      } else {
          error_log($text);
      }
  }
}
}
/**
 * Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

/*
 * ProductAdvertisingAPI
 *
 * https://webservices.amazon.com/paapi5/documentation/index.html
 * https://webservices.amazon.com/paapi5/documentation/get-items.html
 */

use Amazon\ProductAdvertisingAPI\v1\ApiException;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\api\DefaultApi;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\GetItemsRequest;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\GetItemsResource;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\PartnerType;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\ProductAdvertisingAPIClientException;
use Amazon\ProductAdvertisingAPI\v1\Configuration;

require_once(__DIR__ . '/vendor/autoload.php'); // change path as needed

/**
 * Returns the array of items mapped to ASIN
 *
 * @param array $items Items value.
 * @return array of \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\Item mapped to ASIN.
 */
function parseResponse($items)
{
    $mappedResponse = [];
    foreach ($items as $item) {
        $mappedResponse[$item->getASIN()] = $item;
    }
    return $mappedResponse;
}

/**
 * bt-webdesign
 * Anpassungen von Function getItems()
 *
 * config-Paramter
 *
 * Return Array:
 * error = true|false
 *
 * error == true :
 * msg = Text mit Fehlermeldung
 *
 * error == false :
 * asin_product_url = AMAZON-Affiliate-Link
 * asin_title = AMAZON Produkttitel
 * asin_image_small_url, asin_image_small_width, asin_image_small_height
 * asin_image_medium_url, asin_image_medium_width, asin_image_medium_height
 * asin_image_large_url, asin_image_large_width, asin_image_large_height
 */
function getASINInfo($asin)
{
    $config = new Configuration();
    $config->setAccessKey(BT_AMAZON_ACCESSKEY);
    $config->setSecretKey(BT_AMAZON_SECRETKEY);
    $partnerTag = BT_AMAZON_PARTNERTAG;

    $config->setHost(BT_AMAZON_HOST);
    $config->setRegion(BT_AMAZON_REGION);

    $apiInstance = new DefaultApi(
        new GuzzleHttp\Client(),
        $config
    );

    $itemIds = [$asin];

    $resources = ["Images.Primary.Small","Images.Primary.Medium","Images.Primary.Large","ItemInfo.Title"];

    $getItemsRequest = new GetItemsRequest();
    $getItemsRequest->setItemIds($itemIds);
    $getItemsRequest->setPartnerTag($partnerTag);
    $getItemsRequest->setPartnerType(PartnerType::ASSOCIATES);
    $getItemsRequest->setResources($resources);

    $invalidPropertyList = $getItemsRequest->listInvalidProperties();
    $length = count($invalidPropertyList);
    if ($length > 0) {
        $msg = "Error forming the request :";
        foreach ($invalidPropertyList as $invalidProperty) {
            $msg .= $invalidProperty .', ';
        }
        return array( 'error'=> true, 'msg'=>$msg );
    }

    try {
        $getItemsResponse = $apiInstance->getItems($getItemsRequest);

        if ($getItemsResponse->getItemsResult() !== null) {

            if ($getItemsResponse->getItemsResult()->getItems() !== null) {
                $responseList = parseResponse($getItemsResponse->getItemsResult()->getItems());

                foreach ($itemIds as $itemId) {
                    $item = $responseList[$itemId];

                    if ($item !== null && $item->getASIN() && $asin == $item->getASIN() ) {

                           if ($item->getDetailPageURL() !== null && $item->getImages() !== null) {

                              $title = '';
                              if ($item->getItemInfo() !== null and $item->getItemInfo()->getTitle() !== null
                                  and $item->getItemInfo()->getTitle()->getDisplayValue() !== null) {
                                  $title = $item->getItemInfo()->getTitle()->getDisplayValue();
                              }

                              return array( 'error'=> false, 'msg'=>'', 'asin_product_url'=>$item->getDetailPageURL(), 'asin_title'=> $title,
                                     'asin_image_small_url'=>$item->getImages()->getPrimary()->getSmall()->getURL(), 'asin_image_small_width'=>$item->getImages()->getPrimary()->getSmall()->getWidth(), 'asin_image_small_height'=>$item->getImages()->getPrimary()->getSmall()->getHeight(),
                                     'asin_image_medium_url'=>$item->getImages()->getPrimary()->getMedium()->getURL(), 'asin_image_medium_width'=>$item->getImages()->getPrimary()->getMedium()->getWidth(), 'asin_image_medium_height'=>$item->getImages()->getPrimary()->getMedium()->getHeight(),
                                     'asin_image_large_url'=>$item->getImages()->getPrimary()->getLarge()->getURL(), 'asin_image_large_width'=>$item->getImages()->getPrimary()->getLarge()->getWidth(), 'asin_image_large_height'=>$item->getImages()->getPrimary()->getLarge()->getHeight()
                                     );
                           } else {
                              return array( 'error'=> true, 'msg'=>'URLs not found' );
                           }

                    } else {
                       return array( 'error'=> true, 'msg'=>'ASIN not found' );
                    }
                }
            }
        }
        if ($getItemsResponse->getErrors() !== null) {
            return array( 'error'=> true, 'msg'=>'getItemsResponse->getErrors. Error code: '.$getItemsResponse->getErrors()[0]->getCode() .' Error message: '.$getItemsResponse->getErrors()[0]->getMessage() , 'asin_product_url'=>'', 'asin_image_url'=>'', 'asin_title'=>'' );
        }

    } catch (ApiException $exception) {
        return array( 'error'=> true, 'status_code'=>$exception->getCode(), 'msg'=>'Error calling PA-API 5.0. HTTP Status Code: '. $exception->getCode() .' Error Message: '. $exception->getMessage() );
    } catch (Exception $exception) {
        return array( 'error'=> true, 'msg'=>'Error Message: '. $exception->getMessage() );
    }
}